@@ -260,13 +260,13 @@ BUGSdeclClass$methods(
                 code[[3]][[1]] != "I"))
                 stop(
                     paste0('Improper syntax for stochastic declaration: ',
-                           deparse(code))
+                           safeDeparse(code))
                 )
         } else if(code[[1]] == '<-') {
             type <<- 'determ'
         } else {
             stop(paste0('Improper syntax for declaration: ',
-                        deparse(code))
+                        safeDeparse(code))
                  )
         }
         
@@ -296,7 +296,7 @@ BUGSdeclClass$methods(
                     ## There are subscripts inside the transformation
                     if(targetNodeExpr[[1]] != '[') {
                         print(paste("Invalid subscripting for",
-                                    deparse(targetExpr)))
+                                    safeDeparse(targetExpr)))
                     }
                     indexExpr <<- as.list(targetNodeExpr[-c(1,2)])
                     targetVarExpr <<- targetNodeExpr[[2]]
@@ -310,8 +310,8 @@ BUGSdeclClass$methods(
             targetNodeExpr <<- targetVarExpr
         }
         
-        targetVarName <<- deparse(targetVarExpr)
-        targetNodeName <<- deparse(targetNodeExpr)
+        targetVarName <<- safeDeparse(targetVarExpr, warn = TRUE)
+        targetNodeName <<- safeDeparse(targetNodeExpr, warn = TRUE)
     }
 )
 
@@ -377,7 +377,7 @@ makeIndexNamePieces <- function(indexCode) {
     ## Diagnostic for messed up indexing here
     if(as.character(indexCode[[1]] != ':'))
         stop(paste0("Error processing model: something is wrong with the index ",
-                    deparse(indexCode),
+                    safeDeparse(indexCode),
                     ".\nIndexing in model code requires this syntax: '(start expression):(end expression)'."),
              call. = FALSE)
     p1 <- indexCode[[2]]
@@ -465,7 +465,7 @@ BUGSdeclClass$methods(
             )
         if(inherits(targetIndexNamePieces, 'try-error'))
             stop(paste('Error occurred defining ',
-                       deparse(targetExprReplaced)),
+                       safeDeparse(targetExprReplaced)),
                  call. = FALSE)
     if(!nimbleOptions()$allowDynamicIndexing) {
         parentIndexNamePieces <<-
@@ -591,13 +591,13 @@ BUGSdeclClass$methods(
                        is.numeric(boundExprs$upper_) &&
                        boundExprs$lower_ >= boundExprs$upper_)
                         warning(paste0("Lower bound is greater than or equal to upper bound in ",
-                                       deparse(codeReplaced),
+                                       safeDeparse(codeReplaced),
                                        "; proceeding anyway, but this is likely to cause numerical issues."))
                     if(is.numeric(boundExprs$lower_) &&
                        is.numeric(distRange$lower) &&
                        boundExprs$lower_ < distRange$lower) {
                         warning(paste0("Lower bound is less than or equal to distribution lower bound in ",
-                                       deparse(codeReplaced), "; ignoring user-provided lower bound."))
+                                       safeDeparse(codeReplaced), "; ignoring user-provided lower bound."))
                         boundExprs$lower_ <<- distRange$lower
                         codeReplaced[[3]]['lower_'] <<- distRange$lower
                     }
@@ -605,7 +605,7 @@ BUGSdeclClass$methods(
                        is.numeric(distRange$upper) &&
                        boundExprs$upper_ > distRange$upper) {
                         warning(paste0("Upper bound is greater than or equal to distribution upper bound in ",
-                                       deparse(codeReplaced),
+                                       safeDeparse(codeReplaced),
                                        "; ignoring user-provided upper bound."))
                         boundExprs$upper_ <<- distRange$upper
                         codeReplaced[[3]]['upper_'] <<- distRange$upper
@@ -762,7 +762,7 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
             ## error if it looks like mu[i][j] where i is a for-loop index
             if(variable$hasIndex)
                 stop('Error: Variable',
-                     deparse(code[[2]]),
+                     safeDeparse(code[[2]]),
                      'on outside of [ contains a BUGS code index.')
             
             if(variable$replaceable) {
@@ -770,7 +770,7 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                 ## block[i], so block is replaceable
                 if(!all(contentsReplaceable)) 
                     ## dynamic index on a constant
-                    stop('getSymbolicParentNodesRecurse: dynamic indexing of constants is not allowed in ', deparse(code), '. Try adding the dynamically-indexed constant as data instead (using the data argument of nimbleModel).')
+                    stop('getSymbolicParentNodesRecurse: dynamic indexing of constants is not allowed in ', safeDeparse(code), '. Try adding the dynamically-indexed constant as data instead (using the data argument of nimbleModel).')
                 boolIndexingBlock <-
                     unlist(
                         lapply(code[-c(1,2)],
@@ -802,7 +802,7 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                 } else { ## non-replaceable indices are dynamic indices (or constant vectors, which are not allowed)
                     if(!nimbleOptions()$allowDynamicIndexing) {
                         warning("It appears you are trying to use dynamic indexing (i.e., the index of a variable is determined by something that is not a constant) in: ",
-                                deparse(code),
+                                safeDeparse(code),
                                 ". This is now allowed as of version 0.6-6 (as an optional beta feature) and by default as of version 0.6-7. Please set 'nimbleOptions(allowDynamicIndexing = TRUE)' and report any issues to the NIMBLE users group.")
                         dynamicIndexParent <- code[[2]]
                     } else {
@@ -811,8 +811,8 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
                                    detectNonscalarIndex))
                            )
                             stop("getSymbolicParentNodesRecurse: only scalar indices are allowed; vector indexing found in ",
-                                 deparse(code))
-                        indexedVariable <- deparse(code[[2]])
+                                 safeDeparse(code))
+                        indexedVariable <- safeDeparse(code[[2]], warn = TRUE)
                         dynamicIndexParent <-
                             addUnknownIndexToVarNameInBracketExpr(code, contextID)
                         ## Instead of inserting NA, leave indexing
@@ -882,13 +882,15 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
             }
             ## check if the function can be called only in R, not NIMBLE
             isRfunction <- !any(code[[1]] == nimbleFunctionNames)
-            funName <- deparse(code[[1]])
+            funName <- safeDeparse(code[[1]], warn = TRUE)
             isRonly <- isRfunction &
                 (!checkNimbleOrRfunctionNames(funName, envir))
             ## if it can be called only in R but not all contents are replaceable, generate error:
             if(isRonly & !allContentsReplaceable) {
                 if(!exists(funName, envir))
                     stop("R function '", funName,"' does not exist.")
+                if(funName == ":") ## dynamic indexing in a vector of indices
+                    stop("Dynamic indexing found in a vector of indices, ", safeDeparse(code), ". Only scalar indices, such as 'idx' in 'x[idx]', can be dynamic. One can instead use dynamic indexing in a vector of indices inside a nimbleFunction.") 
                 unreplaceable <-
                     sapply(contents[!contentsReplaceable],
                            function(x) as.character(x$code)
@@ -905,7 +907,7 @@ getSymbolicParentNodesRecurse <- function(code, constNames = list(), indexNames
         }
     }
     stop(paste('Something went wrong in getSymbolicVariablesRecurse with',
-               deparse(code)))
+               safeDeparse(code)))
 }
 
 checkNimbleOrRfunctionNames <- function(functionName, envir) {
@@ -1034,7 +1036,7 @@ genReplacementsAndCodeRecurse <- function(code,
         ## or is a nimbleFunction (specifically, an RCfunction)
         if(
         {
-            funName <- deparse(code[[1]])
+            funName <- safeDeparse(code[[1]], warn = TRUE)
             (
                 (funName %in% functionsThatShouldNeverBeReplacedInBUGScode) ||
                 (exists(funName, envir) && is.rcf(get(funName, envir)))
@@ -1055,12 +1057,12 @@ genReplacementsAndCodeRecurse <- function(code,
         isRfunction <- !any(code[[1]] == nimbleFunctionNames)
         isRonly <-
             isRfunction &
-            !checkNimbleOrRfunctionNames(deparse(code[[1]]), envir)
-        if(deparse(code[[1]]) == '$')
+            !checkNimbleOrRfunctionNames(safeDeparse(code[[1]], warn = TRUE), envir)
+        if(safeDeparse(code[[1]], warn = TRUE) == '$')
             isRonly <- FALSE
         if(isRonly & !allContentsReplaceable)
             stop(paste0('Error, R function \"',
-                        deparse(code[[1]]),
+                        safeDeparse(code[[1]]),
                         '\" has non-replaceable node values as arguments.  Must be a nimble function.')
                  )
         if(isRfunction & allContentsReplaceable)
@@ -1072,7 +1074,7 @@ genReplacementsAndCodeRecurse <- function(code,
                                 startingAt=2))
     }
     stop(paste('Something went wrong in genReplacementsAndCodeRecurse with',
-               deparse(code)))
+               safeDeparse(code)))
 }
 
 replaceAllCodeSuccessfully <- function(code) {
