@@ -1,11 +1,10 @@
-
 #' Create an MCMC function from a NIMBLE model, or an MCMC configuration object
 #'
 #' First required argument, which may be of class \code{MCMCconf} (an MCMC configuration object), or inherit from class \code{modelBaseClass} (a NIMBLE model object).  Returns an uncompiled executable MCMC function.  See details.
 #'
 #' @param conf An MCMC configuration object of class \code{MCMCconf} that specifies the model, samplers, monitors, and thinning intervals for the resulting MCMC function.  See \code{configureMCMC} for details of creating MCMC configuration objects.  Alternatively, \code{conf} may a NIMBLE model object, in which case an MCMC function corresponding to the default MCMC configuration for this model is returned.
 #' 
-#' @param ... Additional arguments to be passed to \code{configureMCMC} if \code{conf} is a NIMBLE model object
+#' @param ... Additional arguments to be passed to \code{configureMCMC} if \code{conf} is a NIMBLE model object (see \code{help(configureMCMC)}).
 #'
 #' @details
 #' 
@@ -41,41 +40,7 @@
 #'
 #' @section Calculating WAIC:
 #' 
-#' After the MCMC has been run, calling the \code{calculateWAIC()} method of the MCMC object will return the WAIC for the model, calculated using the posterior samples from the MCMC run.
-#' 
-#' \code{calculateWAIC()} accepts a single arugment:
-#'
-#' \code{nburnin}: The number of pre-thinning MCMC samples to remove from the beginning of the posterior samples for WAIC calculation (default = 0). These samples are discarded in addition to any burn-in specified when running the MCMC.
-#' 
-#' The \code{calculateWAIC} method can only be used if the \code{enableWAIC} 
-#' argument to \code{configureMCMC} or to \code{buildMCMC} is set to \code{TRUE}, or if the NIMBLE option
-#' \code{enableWAIC} is set to \code{TRUE}.  If a user attempts
-#' to call \code{calculateWAIC} without having set \code{enableWAIC = TRUE}
-#' (either in the call to \code{configureMCMC}, or \code{buildMCMC}, or as a NIMBLE option),
-#' an error will occur.  
-#' 
-#' The \code{calculateWAIC} method calculates the WAIC of the model that the
-#' MCMC was performed on. The WAIC (Watanabe, 2010) is calculated from
-#' Equations 5, 12, and 13 in Gelman et al. (2014) (i.e., using \emph{p}WAIC2).
-#'
-#' Note that there is not a unique value of WAIC for a model. The current version of
-#' NIMBLE only provides the conditional WAIC, namely the version of WAIC where all
-#' parameters directly involved in the likelihood are treated as \eqn{theta}
-#' for the purposes of Equation 5 from Gelman et al. (2014). As a result, the user
-#' must set the MCMC monitors (via the \code{monitors} argument) to include all stochastic
-#' nodes that are parents of any data nodes; by default the MCMC monitors are only
-#' the top-level nodes of the model. For more detail on the use of different predictive
-#' distributions, see Section 2.5 from Gelman et al. (2014) or Ariyo et al. (2019).
-
-#' Also note that WAIC relies on a partition of the observations, i.e., 'pointwise'
-#' prediction. In NIMBLE the sum over log pointwise predictive density values treats
-#' each data node as contributing a single value to the sum. When a data node is
-#' multivariate, that data node contributes a single value to the sum based on the
-#' joint density of the elements in the node. Note that if one wants the WAIC
-#' calculation to be based on the joint predictive density for each group of observations
-#' (e.g., grouping the observations from each person or unit in a longitudinal
-#' data context), one would need to use a multivariate distribution for the
-#' observations in each group (potentially by writing a user-defined distribution).
+#' Please see \code{help(waic)} for more information.
 #' 
 #' @examples
 #' \dontrun{
@@ -85,15 +50,17 @@
 #'     y ~ dnorm(x, 1)
 #' })
 #' Rmodel <- nimbleModel(code, data = list(y = 0))
-#' conf <- configureMCMC(Rmodel, monitors = c('mu', 'x'))
-#' Rmcmc <- buildMCMC(conf, enableWAIC = TRUE)
+#' conf <- configureMCMC(Rmodel, monitors = c('mu', 'x'), enableWAIC = TRUE)
+#' Rmcmc <- buildMCMC(conf)
 #' Cmodel <- compileNimble(Rmodel)
 #' Cmcmc <- compileNimble(Rmcmc, project=Rmodel)
 #' Cmcmc$run(10000)
 #' samples <- as.matrix(Cmcmc$mvSamples)
 #' samplesAsList <- as.list(Cmcmc$mvSamples)
 #' head(samples)
-#' WAIC <- Cmcmc$calculateWAIC(nburnin = 1000)
+#' waicInfo <- Cmcmc$getWAIC()
+#' waicInfo$WAIC
+#' waicInfo$pWAIC
 #' }
 #'
 #' @seealso \code{\link{configureMCMC}} \code{\link{runMCMC}} \code{\link{nimbleMCMC}}
@@ -110,10 +77,15 @@
 buildMCMC <- nimbleFunction(
     name = 'MCMC',
     setup = function(conf, ...) {
-    	if(inherits(conf, 'modelBaseClass'))   conf <- configureMCMC(conf, ...)
-    	else if(!inherits(conf, 'MCMCconf')) stop('conf must either be a nimbleModel or a MCMCconf object (created by configureMCMC(...) )')
-        dotdotdotArgs <- list(...)
-        enableWAICargument <- if(!is.null(dotdotdotArgs$enableWAIC)) dotdotdotArgs$enableWAIC else nimbleOptions('MCMCenableWAIC')    ## accept enableWAIC argument regardless
+        dotdotdotArgNames <- names(list(...))
+        if(inherits(conf, 'MCMCconf') && ('enableWAIC' %in% dotdotdotArgNames || 'controlWAIC' %in% dotdotdotArgNames))
+            stop("buildMCMC: 'enableWAIC' and 'controlWAIC' can only be given as arguments when running 'buildMCMC' directly on a model object, not on an MCMC configuration object. Instead pass these argument(s) directly to 'configureMCMC'.") 
+        
+        if(inherits(conf, 'modelBaseClass'))   conf <- configureMCMC(conf, ...)
+        else if(!inherits(conf, 'MCMCconf')) stop('conf must either be a nimbleModel or a MCMCconf object (created by configureMCMC(...) )')
+
+        enableWAIC <- conf$enableWAIC
+        
         model <- conf$model
         my_initializeModel <- initializeModel(model)
         mvSaved <- modelValues(model)
@@ -132,20 +104,25 @@ buildMCMC <- nimbleFunction(
         samplerTimes <- c(0,0) ## establish as a vector
         progressBarLength <- 52  ## multiples of 4 only
         progressBarDefaultSetting <- getNimbleOption('MCMCprogressBar')
-        ## WAIC setup:
-        dataNodes <- model$getNodeNames(dataOnly = TRUE)
-        dataNodeLength <- length(dataNodes)
-        sampledNodes <- model$getVarNames(includeLogProb = FALSE, nodes = monitors)
-        sampledNodes <- sampledNodes[sampledNodes %in% model$getVarNames(includeLogProb = FALSE)]
-        paramDeps <- model$getDependencies(sampledNodes, self = FALSE, downstream = TRUE)
-        allVarsIncludingLogProbs <- model$getVarNames(includeLogProb = TRUE)
-        enableWAIC <- enableWAICargument || conf$enableWAIC   ## enableWAIC comes from MCMC configuration, or from argument to buildMCMC
-        if(enableWAIC) {
-            if(dataNodeLength == 0)   stop('WAIC cannot be calculated, as no data nodes were detected in the model.')
-            mcmc_checkWAICmonitors_conditional(model = model, monitors = sampledNodes, dataNodes = dataNodes)
+
+        waicFun <- nimbleFunctionList(waicClass_base)
+        if(enableWAIC && !('online' %in% names(conf$controlWAIC) && !conf$controlWAIC$online)) {
+           waicFun[[1]] <- buildWAIC(model, mvSaved, conf$controlWAIC)
+           onlineWAIC <- waicFun[[1]]$online 
+           thinWAIC <- waicFun[[1]]$thin
+        } else {
+            if(enableWAIC) {  
+                ## Setup for original (offline) WAIC prior to v. 0.12.0, namely cWAIC with no grouping capability.
+                ## Retained for backward compatibility.
+                ## It could also be useful to allow post hoc calculation of mWAIC given its computational
+                ## cost, but that is not implemented.
+                waicFun[[1]] <- buildOfflineWAIC(model, mvSamples, conf$controlWAIC, monitors)
+            } else waicFun[[1]] <- buildDummyWAIC()
+            onlineWAIC <- FALSE
+            thinWAIC <- FALSE
         }
     },
-
+    
     run = function(
         niter                 = integer(),
         reset                 = logical(default = TRUE),
@@ -156,7 +133,8 @@ buildMCMC <- nimbleFunction(
         ##samplerExecutionOrder = integer(1, default = -1)
         nburnin               = integer(default =  0),
         thin                  = integer(default = -1),
-        thin2                 = integer(default = -1)) {
+        thin2                 = integer(default = -1),
+        resetWAIC             = logical(default = TRUE)) {
         if(niter < 0)       stop('cannot specify niter < 0')
         if(nburnin < 0)     stop('cannot specify nburnin < 0')
         if(nburnin > niter) stop('cannot specify nburnin > niter')
@@ -181,6 +159,8 @@ buildMCMC <- nimbleFunction(
                 mvSamples2_copyRow <- getsize(mvSamples2)
             }
         }
+        if(onlineWAIC & resetWAIC)
+            waicFun[[1]]$reset()
         resize(mvSamples,  mvSamples_copyRow  + floor((niter-nburnin) / thinToUseVec[1]))
         resize(mvSamples2, mvSamples2_copyRow + floor((niter-nburnin) / thinToUseVec[2]))
         ## reinstate samplerExecutionOrder as a runtime argument, once we support non-scalar default values for runtime arguments:
@@ -223,6 +203,13 @@ buildMCMC <- nimbleFunction(
                     mvSamples2_copyRow <- mvSamples2_copyRow + 1
                     nimCopy(from = model, to = mvSamples2, row = mvSamples2_copyRow, nodes = monitors2)
                 }
+                if(enableWAIC & onlineWAIC) {
+                    if (!thinWAIC) {
+                        waicFun[[1]]$updateStats()
+                    } else if (sampleNumber %% thinToUseVec[1] == 0){ 
+                        waicFun[[1]]$updateStats()
+                    }
+                }
             }
             if(progressBar & (iter == progressBarNextFloor)) {
                 cat('-')
@@ -238,47 +225,34 @@ buildMCMC <- nimbleFunction(
             returnType(double(1))
             return(samplerTimes[1:(length(samplerTimes)-1)])
         },
-        calculateWAIC = function(nburnin = integer(default = 0),
-            burnIn = integer(default = 0)) {
+        ## Old-style post-sampling WAIC calculation.
+        calculateWAIC = function(nburnin = integer(default = 0)) {
             if(!enableWAIC) {
-                print('Error: must set enableWAIC = TRUE in buildMCMC. See help(buildMCMC) for additional information.')
+                print('Error: One must set enableWAIC = TRUE in \'configureMCMC\' or \'buildMCMC\'. See \'help(configureMCMC)\' for additional information.')
                 return(NaN)
             }
-            if(burnIn != 0) {
-                print('Warning: \'burnIn\' argument is deprecated and will not be supported in future versions of NIMBLE. Please use the \'nburnin\' argument instead.')
-                ## If nburnin has not been changed, replace with burnIn value
-                if(nburnin == 0)   nburnin <- burnIn
-            }
-            nburninPostThinning <- ceiling(nburnin/thinToUseVec[1])
-            numMCMCSamples <- getsize(mvSamples) - nburninPostThinning
-            if((numMCMCSamples) < 2) {
-                print('Error: need more than one post burn-in MCMC samples')
-                return(-Inf)
-            }
-            logPredProbs <- matrix(nrow = numMCMCSamples, ncol = dataNodeLength)
-            logAvgProb <- 0
-            pWAIC <- 0
-            currentVals <- values(model, allVarsIncludingLogProbs)
-            
-            for(i in 1:numMCMCSamples) {
-                copy(mvSamples, model, nodesTo = sampledNodes, row = i + nburninPostThinning)
-                model$simulate(paramDeps)
-                model$calculate(dataNodes)
-                for(j in 1:dataNodeLength)
-                    logPredProbs[i,j] <- model$getLogProb(dataNodes[j])
-            }
-            for(j in 1:dataNodeLength) {
-                maxLogPred <- max(logPredProbs[,j])
-                thisDataLogAvgProb <- maxLogPred + log(mean(exp(logPredProbs[,j] - maxLogPred)))
-                logAvgProb <- logAvgProb + thisDataLogAvgProb
-                pointLogPredVar <- var(logPredProbs[,j])
-                pWAIC <- pWAIC + pointLogPredVar
-            }
-            WAIC <- -2*(logAvgProb - pWAIC)
-            values(model, allVarsIncludingLogProbs) <<- currentVals
-            if(is.nan(WAIC)) print('WAIC was calculated as NaN.  You may need to add monitors to model latent states, in order for a valid WAIC calculation.')
+            result <- waicFun[[1]]$calculateWAIC(nburnin, thinToUseVec[1])
             returnType(double())
-            return(WAIC)
-        })
+            return(result$WAIC)
+        },
+        getWAIC = function() {
+            returnType(waicList())
+            if(enableWAIC) {
+                return(waicFun[[1]]$get())
+            } else {
+                print("WAIC was disabled based on the 'enableWAIC = FALSE'. You may be able to use the \'calculateWAIC\' function.")
+                return(waicList$new(WAIC = NA, lppd = NA, pWAIC = NA))
+            }
+        },
+        getWAICdetails = function(returnElements = logical(default = FALSE)) {
+            returnType(waicDetailsList())
+            if(enableWAIC & onlineWAIC) {
+                return(waicFun[[1]]$getDetails(returnElements))
+            } else {
+                print("WAIC details are only available when using online WAIC. Online WAIC was disabled based on the 'onlineWAIC' element of WAIC control list.")
+                return(waicDetailsList$new(marginal = FALSE, niterMarginal = 0, thin = FALSE, online = FALSE))
+            }
+        }
+    )
+        
 )
-
