@@ -1,3 +1,24 @@
+/*
+ * NIMBLE: an R package for programming with BUGS models.
+ * Copyright (C) 2014-2017 Perry de Valpine, Christopher Paciorek,
+ * Daniel Turek, Clifford Anderson-Bergman, Nick Michaud, Fritz Obermeyer,
+ * Duncan Temple Lang.
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, a copy is available at
+ * https://www.R-project.org/Licenses/
+ */
+
 // Code to be included in each on-the-fly (dynamic) nimble compilation
 // It will be included by a generated file in each project
 #ifndef __DYNAMIC_REGISTRATIONS
@@ -84,6 +105,7 @@ R_CallMethodDef CallEntries[] = {
   // from NamedObjects
   FUN(getModelObjectPtr, 2),
   FUN(getAvailableNames, 1),
+  FUN(copyFromRobject, 2),
   FUN(getNumberedObject, 2),
   FUN(setNumberedObject, 3),
   FUN(resizeNumberedObjects, 2),
